from importlib import import_module
import json

from django.db import models
from django.utils.translation import ugettext_lazy as _

from jet.utils import LazyDateTimeEncoder

try:
    from django.utils.encoding import python_2_unicode_compatible
except ImportError:
    from six import python_2_unicode_compatible


@python_2_unicode_compatible
class UserDashboardModule(models.Model):
    title = models.CharField(verbose_name=_("Title"), max_length=255)
    module = models.CharField(verbose_name=_("module"), max_length=255)
    app_label = models.CharField(
        verbose_name=_("application name"), max_length=255, null=True, blank=True
    )
    user = models.PositiveIntegerField(verbose_name=_("user"))
    column = models.PositiveIntegerField(verbose_name=_("column"))
    order = models.IntegerField(verbose_name=_("order"))
    settings = models.TextField(verbose_name=_("settings"), default="", blank=True)
    children = models.TextField(verbose_name=_("children"), default="", blank=True)
    collapsed = models.BooleanField(verbose_name=_("collapsed"), default=False)

    class Meta:
        verbose_name = _("user dashboard module")
        verbose_name_plural = _("user dashboard modules")
        ordering = ("column", "order")

    def __str__(self):
        return self.module

    def load_module(self):
        try:
            package, module_name = self.module.rsplit(".", 1)
            package = import_module(package)
            module = getattr(package, module_name)

            return module
        except AttributeError:
            return None
        except ImportError:
            return None

    def pop_settings(self, pop_settings):
        settings = json.loads(self.settings)

        for setting in pop_settings:
            if setting in settings:
                settings.pop(setting)

        self.settings = json.dumps(settings, cls=LazyDateTimeEncoder)
        self.save()

    def update_settings(self, update_settings):
        settings = json.loads(self.settings)

        settings.update(update_settings)

        self.settings = json.dumps(settings, cls=LazyDateTimeEncoder)
        self.save()



from functools import reduce
import json
import operator

from django import forms
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db.models import Q

from jet.models import Bookmark, PinnedApplication
from jet.utils import get_model_instance_label, user_is_authenticated

try:
    from django.apps import apps

    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model


class AddBookmarkForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(AddBookmarkForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Bookmark
        fields = ["url", "title"]

    def clean(self):
        data = super(AddBookmarkForm, self).clean()
        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")
        if not self.request.user.has_perm("jet.change_bookmark"):
            raise ValidationError("error")
        return data

    def save(self, commit=True):
        self.instance.user = self.request.user.pk
        return super(AddBookmarkForm, self).save(commit)


class RemoveBookmarkForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(RemoveBookmarkForm, self).__init__(*args, **kwargs)

    class Meta:
        model = Bookmark
        fields = []

    def clean(self):
        data = super(RemoveBookmarkForm, self).clean()
        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")
        if self.instance.user != self.request.user.pk:
            raise ValidationError("error")
        return data

    def save(self, commit=True):
        if commit:
            self.instance.delete()


class ToggleApplicationPinForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(ToggleApplicationPinForm, self).__init__(*args, **kwargs)

    class Meta:
        model = PinnedApplication
        fields = ["app_label"]

    def clean(self):
        data = super(ToggleApplicationPinForm, self).clean()
        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")
        return data

    def save(self, commit=True):
        if commit:
            try:
                pinned_app = PinnedApplication.objects.get(
                    app_label=self.cleaned_data["app_label"], user=self.request.user.pk
                )
                pinned_app.delete()
                return False
            except PinnedApplication.DoesNotExist:
                PinnedApplication.objects.create(
                    app_label=self.cleaned_data["app_label"], user=self.request.user.pk
                )
                return True


class ModelLookupForm(forms.Form):
    app_label = forms.CharField()
    model = forms.CharField()
    q = forms.CharField(required=False)
    page = forms.IntegerField(required=False)
    page_size = forms.IntegerField(required=False, min_value=1, max_value=1000)
    object_id = forms.IntegerField(required=False)
    model_cls = None

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(ModelLookupForm, self).__init__(*args, **kwargs)

    def clean(self):
        data = super(ModelLookupForm, self).clean()

        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")

        try:
            self.model_cls = get_model(data["app_label"], data["model"])
        except:
            raise ValidationError("error")

        content_type = ContentType.objects.get_for_model(self.model_cls)
        permission = Permission.objects.filter(
            content_type=content_type, codename__startswith="change_"
        ).first()

        if not self.request.user.has_perm(
            "{}.{}".format(data["app_label"], permission.codename)
        ):
            raise ValidationError("error")

        return data

    def lookup(self):
        qs = self.model_cls.objects

        if self.cleaned_data["q"]:
            if getattr(self.model_cls, "autocomplete_search_fields", None):
                search_fields = self.model_cls.autocomplete_search_fields()
                filter_data = [
                    Q((field + "__icontains", self.cleaned_data["q"]))
                    for field in search_fields
                ]
                # if self.cleaned_data['object_id']:
                #     filter_data.append(Q(pk=self.cleaned_data['object_id']))
                qs = qs.filter(reduce(operator.or_, filter_data)).distinct()
            else:
                qs = qs.none()

        limit = self.cleaned_data["page_size"] or 100
        page = self.cleaned_data["page"] or 1
        offset = (page - 1) * limit

        items = list(
            map(
                lambda instance: {
                    "id": instance.pk,
                    "text": get_model_instance_label(instance),
                },
                qs.all()[offset : offset + limit],
            )
        )
        total = qs.count()

        return items, total



import django
from django.conf.urls import url

try:
    from django.views.i18n import JavaScriptCatalog

    javascript_catalog = JavaScriptCatalog.as_view()
except ImportError:  # Django < 2.0
    from django.views.i18n import javascript_catalog

from jet.views import (
    add_bookmark_view,
    model_lookup_view,
    remove_bookmark_view,
    toggle_application_pin_view,
)

app_name = "jet"

urlpatterns = [
    url(r"^add_bookmark/$", add_bookmark_view, name="add_bookmark"),
    url(r"^remove_bookmark/$", remove_bookmark_view, name="remove_bookmark"),
    url(
        r"^toggle_application_pin/$",
        toggle_application_pin_view,
        name="toggle_application_pin",
    ),
    url(r"^model_lookup/$", model_lookup_view, name="model_lookup"),
    url(
        r"^jsi18n/$",
        javascript_catalog,
        {"packages": "django.contrib.admin+jet"},
        name="jsi18n",
    ),
]

if django.VERSION[:2] < (1, 8):
    from django.conf.urls import patterns

    urlpatterns = patterns("", *urlpatterns)



import django
from django.conf.urls import url

try:
    from django.views.i18n import JavaScriptCatalog

    javascript_catalog = JavaScriptCatalog.as_view()
except ImportError:  # Django < 2.0
    from django.views.i18n import javascript_catalog

from jet.dashboard import dashboard
from jet.dashboard.views import (
    UpdateDashboardModuleView,
    add_user_dashboard_module_view,
    load_dashboard_module_view,
    remove_dashboard_module_view,
    reset_dashboard_view,
    update_dashboard_module_collapse_view,
    update_dashboard_modules_view,
)

app_name = "dashboard"

urlpatterns = [
    url(
        r"^module/(?P<pk>\d+)/$",
        UpdateDashboardModuleView.as_view(),
        name="update_module",
    ),
    url(
        r"^update_dashboard_modules/$",
        update_dashboard_modules_view,
        name="update_dashboard_modules",
    ),
    url(
        r"^add_user_dashboard_module/$",
        add_user_dashboard_module_view,
        name="add_user_dashboard_module",
    ),
    url(
        r"^update_dashboard_module_collapse/$",
        update_dashboard_module_collapse_view,
        name="update_dashboard_module_collapse",
    ),
    url(
        r"^remove_dashboard_module/$",
        remove_dashboard_module_view,
        name="remove_dashboard_module",
    ),
    url(
        r"^load_dashboard_module/(?P<pk>\d+)/$",
        load_dashboard_module_view,
        name="load_dashboard_module",
    ),
    url(r"^reset_dashboard/$", reset_dashboard_view, name="reset_dashboard"),
    url(r"^jsi18n/$", javascript_catalog, {"packages": "jet"}, name="jsi18n"),
]

urlpatterns += dashboard.urls.get_urls()

if django.VERSION[:2] < (1, 8):
    from django.conf.urls import patterns

    urlpatterns = patterns("", *urlpatterns)



import json

from django import forms
from django.core.exceptions import ValidationError

from jet.dashboard.models import UserDashboardModule
from jet.dashboard.utils import get_current_dashboard
from jet.utils import user_is_authenticated


class UpdateDashboardModulesForm(forms.Form):
    app_label = forms.CharField(required=False)
    modules = forms.CharField()
    modules_objects = []

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(UpdateDashboardModulesForm, self).__init__(*args, **kwargs)

    def clean(self):
        data = super(UpdateDashboardModulesForm, self).clean()

        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")

        try:
            modules = json.loads(data["modules"])

            for module in modules:
                db_module = UserDashboardModule.objects.get(
                    user=self.request.user.pk,
                    app_label=data["app_label"] if data["app_label"] else None,
                    pk=module["id"],
                )

                column = module["column"]
                order = module["order"]

                if db_module.column != column or db_module.order != order:
                    db_module.column = column
                    db_module.order = order

                    self.modules_objects.append(db_module)
        except Exception:
            raise ValidationError("error")

        return data

    def save(self):
        for module in self.modules_objects:
            module.save()


class AddUserDashboardModuleForm(forms.ModelForm):
    type = forms.CharField()
    module = forms.IntegerField()
    module_cls = None

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(AddUserDashboardModuleForm, self).__init__(*args, **kwargs)

    class Meta:
        model = UserDashboardModule
        fields = ["app_label"]

    def clean_app_label(self):
        data = self.cleaned_data["app_label"]
        return data if data != "" else None

    def clean(self):
        data = super(AddUserDashboardModuleForm, self).clean()

        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")

        if "app_label" in data:
            index_dashboard_cls = get_current_dashboard(
                "app_index" if data["app_label"] else "index"
            )
            index_dashboard = index_dashboard_cls(
                {"request": self.request}, app_label=data["app_label"]
            )

            if "type" in data:
                if data["type"] == "children":
                    module = index_dashboard.children[data["module"]]
                elif data["type"] == "available_children":
                    module = index_dashboard.available_children[data["module"]]()
                else:
                    raise ValidationError("error")

                self.module_cls = module
        return data

    def save(self, commit=True):
        self.instance.title = self.module_cls.title
        self.instance.module = self.module_cls.fullname()
        self.instance.user = self.request.user.pk
        self.instance.column = 0
        self.instance.order = -1
        self.instance.settings = self.module_cls.dump_settings()
        self.instance.children = self.module_cls.dump_children()

        return super(AddUserDashboardModuleForm, self).save(commit)


class UpdateDashboardModuleCollapseForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(UpdateDashboardModuleCollapseForm, self).__init__(*args, **kwargs)

    class Meta:
        model = UserDashboardModule
        fields = ["collapsed"]

    def clean(self):
        data = super(UpdateDashboardModuleCollapseForm, self).clean()

        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")

        if self.instance.user != self.request.user.pk:
            raise ValidationError("error")

        return data


class RemoveDashboardModuleForm(forms.ModelForm):
    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(RemoveDashboardModuleForm, self).__init__(*args, **kwargs)

    class Meta:
        model = UserDashboardModule
        fields = []

    def clean(self):
        cleaned_data = super(RemoveDashboardModuleForm, self).clean()

        if (
            not user_is_authenticated(self.request.user)
            or self.instance.user != self.request.user.pk
        ):
            raise ValidationError("error")

        return cleaned_data

    def save(self, commit=True):
        if commit:
            self.instance.delete()


class ResetDashboardForm(forms.Form):
    app_label = forms.CharField(required=False)

    def __init__(self, request, *args, **kwargs):
        self.request = request
        super(ResetDashboardForm, self).__init__(*args, **kwargs)

    class Meta:
        model = UserDashboardModule
        fields = []

    def clean(self):
        data = super(ResetDashboardForm, self).clean()
        data["app_label"] = data["app_label"] if data["app_label"] else None

        if (
            not user_is_authenticated(self.request.user)
            or not self.request.user.is_staff
        ):
            raise ValidationError("error")

        return data

    def save(self, commit=True):
        if commit:
            UserDashboardModule.objects.filter(
                user=self.request.user.pk, app_label=self.cleaned_data["app_label"]
            ).delete()
